import ast
import error

from ast import Add, BinOp, Constant, Expression, Mult, Name, Sub, expr
from collections.abc import Mapping
from dataclasses import dataclass

type Value = IntValue | StrValue


@dataclass
class IntValue:
    value: int


@dataclass
class StrValue:
    value: str


def eval_expr(e: expr, m: Mapping[str, int]) -> int:
    try:
        match e:
            case Name(id):
                if id in m:
                    return m[id]
                else:
                    raise NameError("name '" + id + "' is not defined")
            case Constant(i) if type(i) is int:
                return i
            case BinOp(left, Add(), right):
                left_value: int = eval_expr(left, m)
                right_value: int = eval_expr(right, m)
                return left_value + right_value
            case BinOp(left, Sub(), right):
                left_value: int = eval_expr(left, m)
                right_value: int = eval_expr(right, m)
                return left_value - right_value
            case BinOp(left, Mult(), right):
                left_value: int = eval_expr(left, m)
                right_value: int = eval_expr(right, m)
                return left_value * right_value
            case _:
                raise ValueError("unsupported expression")
    except Exception as err:
        error.fail(e, err)


def eval(e: Expression, m: Mapping[str, int]) -> int:
    return eval_expr(e.body, m)


src = """
(3 + (6 * 5)) * (z * 7)
"""


def main() -> None:
    e: Expression = ast.parse(src, mode='eval')
    print(ast.dump(e, indent=4))
    print(src)
    print(eval(e, {}))


main()
